/*
 *   $Id: gram.y,v 1.1 1996/11/12 23:20:50 lf Exp $
 *
 *   Authors:
 *    Pedro Roque		<roque@di.fc.ul.pt>
 *    Lars Fenneberg		<lf@elemental.net>	 
 *
 *   This software is Copyright 1996 by the above mentioned author(s), 
 *   All Rights Reserved.
 *
 *   The license which is distributed with this software in the file COPYRIGHT
 *   applies to this software. If your distribution is missing this file, you
 *   may request it from <lf@elemental.net>.
 *
 */
%{
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <malloc.h>
#include <syslog.h>
#include <netinet6/in6.h>

#include "radv.h"
#include "defaults.h"

extern struct Interface *IfaceList;
struct Interface *iface = NULL;
struct AdvPrefix *prefix = NULL;

extern char *conf_file;
extern int num_lines;
extern char *yytext;

static void iface_init_defaults(struct Interface *);
static int palloc_check(void);

static void cleanup(void);
static void yyerror(char *msg);

#define ABORT	do { cleanup(); YYABORT; } while (0);

%}

%token		T_INTERFACE
%token		T_PREFIX
%token		T_LEN

%token	<str>	STRING
%token	<num>	NUMBER
%token	<bool>	SWITCH
%token	<addr>	ADDR

%token		T_AdvSendAdvert
%token		T_MaxRtrAdvInterval
%token		T_MinRtrAdvInterval
%token		T_AdvManagedFlag
%token		T_AdvOtherConfigFlag
%token		T_AdvLinkMTU
%token		T_AdvReachableTime
%token		T_AdvRetransTimer
%token		T_AdvCurHopLimit
%token		T_AdvDefaultLifetime
%token		T_AdvSourceLLAddress

%token		T_AdvOnLink
%token		T_AdvAutonomous
%token		T_AdvValidLifetime
%token		T_AdvPreferedLifetime

%token		T_BAD_TOKEN

%type	<str>	name
%type	<pinfo> prefixdef prefixlist

%union {
	int			num;
	int			bool;
	struct in6_addr		*addr;
	char			*str;
	struct AdvPrefix	*pinfo;
};

%%

grammar		: grammar ifacedef
		| ifacedef
		;

ifacedef	: T_INTERFACE name '{' ifaceparams  '}' ';'
		{
			strcpy(iface->Name, $2);

			iface->next = IfaceList;
			IfaceList = iface;

			dlog(LOG_DEBUG, 4, "interface definition for %s is ok", iface->Name);

			iface = NULL;
		};
	
name		: STRING
		{
			/* check vality */
			$$ = $1;
		}
		;

ifaceparams	: iface_advt optional_ifacevlist prefixlist
		{
			iface->AdvPrefixList = $3;
		}
		;

optional_ifacevlist: /* empty */
		   | ifacevlist
		   ;

ifacevlist	: ifacevlist ifaceval
		| ifaceval
		;

iface_advt	: T_AdvSendAdvert SWITCH ';'
		{
			iface = malloc(sizeof(struct Interface));

			if (iface == NULL) {
				log(LOG_CRIT, "malloc failed: %s", strerror(errno));
				ABORT;
			}

			iface_init_defaults(iface);
			iface->AdvSendAdvert = $2;
		}

ifaceval	: T_MinRtrAdvInterval NUMBER ';'
		{
			if (($2 < MIN_MinRtrAdvInterval) || 
				($2 > MAX_MinRtrAdvInterval(iface))) {
				log(LOG_ERR, 
					"MinRtrAdvInterval must be between %d and %d in %s,line %d",
					MIN_MinRtrAdvInterval,  MAX_MinRtrAdvInterval(iface),
					conf_file, num_lines);
				ABORT;
			}

			iface->MinRtrAdvInterval = $2;
		}
		| T_MaxRtrAdvInterval NUMBER ';'
		{
			if (($2 < MIN_MaxRtrAdvInterval) || ($2 > MAX_MaxRtrAdvInterval)) {
				log(LOG_ERR, 
					"MaxRtrAdvInterval must be between %d and %d in %s,line %d",
					MIN_MaxRtrAdvInterval, MAX_MaxRtrAdvInterval, 
					conf_file, num_lines);
				ABORT;
			}

			iface->MaxRtrAdvInterval = $2;
		}
		| T_AdvManagedFlag SWITCH ';'
		{
			iface->AdvManagedFlag = $2;
		}
		| T_AdvOtherConfigFlag SWITCH ';'
		{
			iface->AdvOtherConfigFlag = $2;
		}
		| T_AdvLinkMTU NUMBER ';'
		{
			if (($2 != 0) && ($2 < MIN_AdvLinkMTU)) {
				log(LOG_ERR, 
					"AdvLinkMTU must be zero or greater than %d in %s,line %d",
					MIN_AdvLinkMTU, conf_file, num_lines);
				ABORT;
			}

			iface->AdvLinkMTU = $2;
		}
		| T_AdvReachableTime NUMBER ';'
		{
			if ($2 >  MAX_AdvReachableTime) {
				log(LOG_ERR, 
					"AdvReachableTime must be less than %d in %s,line %d",
					MAX_AdvReachableTime, conf_file, num_lines);
				ABORT;
			}

			iface->AdvReachableTime = $2;
		}
		| T_AdvRetransTimer NUMBER ';'
		{
			iface->AdvRetransTimer = $2;
		}
		| T_AdvDefaultLifetime NUMBER ';'
		{
			iface->AdvDefaultLifetime = $2;
		}
		| T_AdvCurHopLimit NUMBER ';'
		{
			if ($2 > MAX_AdvCurHopLimit)
			{
				log(LOG_ERR, "AdvCurHopLimit must not be greater than %d in"
					" %s, line %d", MAX_AdvCurHopLimit, conf_file, num_lines);
				ABORT;
			}
			iface->AdvCurHopLimit = $2;
		}
		| T_AdvSourceLLAddress SWITCH ';'
		{
			iface->AdvSourceLLAddress = $2;
		}
		;
		
prefixlist	: prefixdef
		{
			$$ = $1;
		}
		| prefixlist prefixdef
		{
			$2->next = $1;
			$$ = $2;
		}
		;

prefixdef	: T_PREFIX ADDR T_LEN NUMBER '{' optional_prefixplist '}' ';'
		{
			if (palloc_check() < 0)
				ABORT;

			if ($4 > MAX_PrefixLen)
			{
				log(LOG_ERR, "invalid prefix length in %s, line %d", conf_file, num_lines);
				ABORT;
			}
			

			prefix->PrefixLen = $4;

			if (prefix->AdvPreferedLifetime >
			    prefix->AdvValidLifetime)
			{
				log(LOG_ERR, "AdvValidLifeTime must be "
					"greater than AdvPreferedLifetime in %s, line %d", 
					conf_file, num_lines);
				ABORT;
			}

			memcpy(&prefix->Prefix, $2, sizeof(struct in6_addr));
			
			$$ = prefix;
			prefix = NULL;
		}
		;

optional_prefixplist: /* empty */
		    | prefixplist 

prefixplist	: prefixplist prefixparms
		| prefixparms
		;

prefixparms	: T_AdvOnLink SWITCH ';'
		{
			palloc_check();
			prefix->AdvOnLinkFlag = $2;
		}
		| T_AdvAutonomous SWITCH ';'
		{
			palloc_check();
			prefix->AdvAutonomousFlag = $2;
		}
		| T_AdvValidLifetime NUMBER ';'
		{
			palloc_check();
			prefix->AdvValidLifetime = $2;
		}
		| T_AdvPreferedLifetime NUMBER ';'
		{
			palloc_check();
			prefix->AdvPreferedLifetime = $2;
		}
		;

%%

static
void cleanup(void)
{
	if (iface)
		free(iface);
	
	if (prefix)
		free(prefix);
}

static void
yyerror(char *msg)
{
	cleanup();
	log(LOG_ERR, "%s in %s,line %d: %s", msg, conf_file, num_lines, yytext);
}

static void
iface_init_defaults(struct Interface *iface)
{
	memset(iface, 0, sizeof(struct Interface));
	iface->MaxRtrAdvInterval  = DFLT_MaxRtrAdvInterval;
	iface->MinRtrAdvInterval  = DFLT_MinRtrAdvInterval;
	iface->AdvDefaultLifetime = DFLT_AdvDefaultLifetime;
	iface->AdvLinkMTU	  = DFLT_AdvLinkMTU;
	iface->AdvSourceLLAddress = DFLT_AdvSourceLLAddress;
}

static int
palloc_check(void)
{
	if (prefix == NULL)
	{
		prefix = malloc(sizeof(struct AdvPrefix));
		
		if (prefix == NULL) {
			log(LOG_CRIT, "malloc failed: %s", strerror(errno));
			return (-1);
		}

		memset(prefix, 0, sizeof(struct AdvPrefix));
		
		prefix->AdvOnLinkFlag = DFLT_AdvOnLinkFlag;
		prefix->AdvAutonomousFlag = DFLT_AdvAutonomousFlag;
		prefix->AdvValidLifetime = DFLT_AdvValidLifetime;
		prefix->AdvPreferedLifetime = DFLT_AdvPreferedLifetime;
	}

	return 0;
}
