/*
 *   $Id: radv.c,v 1.1 1996/11/12 23:20:50 lf Exp $
 *
 *   Authors:
 *    Pedro Roque		<roque@di.fc.ul.pt>
 *    Lars Fenneberg		<lf@elemental.net>	 
 *
 *   This software is Copyright 1996 by the above mentioned author(s), 
 *   All Rights Reserved.
 *
 *   The license which is distributed with this software in the file COPYRIGHT
 *   applies to this software. If your distribution is missing this file, you
 *   may request it from <lf@elemental.net>.
 *
 */

#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <getopt.h>
#include <string.h>
#include <signal.h>
#include <syslog.h>
#include <stdlib.h>

#include <sys/types.h>

#include "radv.h"
#include "pathnames.h"

struct Interface *IfaceList;

char usage[] = "[-v] [-d level] [-C config_file] [-m log_method] [-l log_file]";

struct option prog_opt[] = {
	{"debug", 1, 0, 'd'},
	{"config", 1, 0, 'C'},
	{"logfile", 1, 0, 'l'},
	{"logmethod", 1, 0, 'm'},
	{"version", 0, 0, 'v'},
	{NULL, 0, 0, 0}
};

extern FILE *yyin;

char *conf_file = NULL;
char *pname;
int sock = -1;

void sighup_handler(int sig);
void timer_handler(void *data);
int readin_config(char *);
void version(void);

int
main(int argc, char *argv[])
{
	unsigned char msg[2048];
	struct Interface *iface;
	int opt_idx;
	int c, log_method;
	char *opt_l;

	pname = ((pname=strrchr(argv[0],'/')) != NULL)?pname+1:argv[0];

	log_method = L_SYSLOG;
	opt_l = NULL;
	conf_file = PATH_RADV_CONF;

	/* parse args */
	while ((c = getopt_long(argc, argv, "d:C:l:m:v", prog_opt, &opt_idx)) > 0)
	{
		switch (c) {
		case 'C':
			conf_file = optarg;
			break;
		case 'd':
			set_debuglevel(atoi(optarg));
			break;
		case 'l':
			opt_l = optarg;
			break;
		case 'm':
			if (!strcmp(optarg, "syslog")) {
				log_method = L_SYSLOG;
			} else if (!strcmp(optarg, "stderr")) {
				log_method = L_STDERR;
			} else if (!strcmp(optarg, "logfile")) {
				log_method = L_LOGFILE;
			} else if (!strcmp(optarg, "none")) {
				log_method = L_NONE;
			} else {
				fprintf(stderr, "%s: unknown log method: %s\n", pname, optarg);
				exit(1);
			}
			break;
		case 'v':
			version();
			break;
		case ':':
			fprintf(stderr, "%s: option %s: parameter expected\n", pname,
				prog_opt[opt_idx].name);
			exit(1);
		case '?':
			exit(1);
		}
	}
	
	if ((log_method == L_LOGFILE) && (opt_l == NULL))
		opt_l = PATH_RADV_LOG;

	if (log_open(log_method, pname, opt_l) < 0)
		exit(1);

	log(LOG_INFO, "version %s started", VERSION);

	/* get a raw socket for sending and receiving ICMPv6 messages */
	sock = open_icmpv6_socket();
	if (sock < 0)
		exit(1);

	/* parse config file */
	if (readin_config(conf_file) < 0)
		exit(1);

	/*
	 *	setup link local address
	 */
	for(iface=IfaceList; iface; iface=iface->next)
		if (setup_linklocal_addr(iface) < 0)
			exit(1);
	/*
	 *	setup link layer address
	 */
	for(iface=IfaceList; iface; iface=iface->next)
		if (setup_linklayer_addr(sock, iface) < 0)
			exit(1);

	/*
	 * okay, config file is read in, socket and stuff is setup, so
	 * lets fork now...
	 */

	if (get_debuglevel() == 0) {

		switch (fork()) {
			case 0:
				/* child falls through */
				break;
			case -1:
				log(LOG_ERR, "fork: %s", strerror(errno));
				exit(1);
			default:
				/* parent exists */
				log_close();
				exit(0);	
		}

		/*
		 * reopen logfile, so that we get the process id right in the syslog
		 */
		log_close();
		if (log_open(log_method, pname, opt_l) < 0)
			exit(1);

		/*
		 * detach from our controlling terminal
		 */
	 
		setsid();
	}

	/*
	 *	config signal handlers
	 */

	signal(SIGHUP, sighup_handler);

	/*
	 *	send initial advertisement and set timers
	 */

	for(iface=IfaceList; iface; iface=iface->next) {

		/* send an initial advertisement */
		send_ra(sock, iface, NULL);
		
		memset(&iface->tm, 0, sizeof(struct timer_lst));
		iface->tm.handler = timer_handler;
		iface->tm.data = (void *) iface;
		set_timer(&iface->tm, iface->MaxRtrAdvInterval);
	}


	/* enter loop */

	for (;;)
	{
		int len;
		struct sockaddr_in6 rcv_addr;
		struct in6_pktinfo *pkt_info = NULL;
		
		len = recv_rs(sock, msg, &rcv_addr, &pkt_info);
		if (len > 0)
			process_msg(sock, IfaceList, msg, len, 
				&rcv_addr, pkt_info);
	}
	
}

void
timer_handler(void *data)
{
	struct Interface *iface = (struct Interface *) data;

	dlog(LOG_DEBUG, 4, "timer_handler called for %s", iface->Name);

	send_ra(sock, iface, NULL);
	set_timer(&iface->tm, iface->MaxRtrAdvInterval);
}

void
sighup_handler(int sig)
{
	/* FIXME: reread config file here.... */

	/* Linux has "one-shot" signals, reinstall the signal handler */
	signal(SIGHUP, sighup_handler);
}

int
readin_config(char *fname)
{
	if ((yyin = fopen(fname, "r")) == NULL) {
		log(LOG_ERR, "can't open %s: %s", fname, strerror(errno));
		return (-1);
	}

	if (yyparse() != 0)
		return (-1);

	return 0;
}

void
version(void)
{
	fprintf(stderr,"%s: version %s\n", pname, VERSION);
	exit(1);	
}
